package com.mysql.jdbc;

import java.sql.SQLException;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.Calendar;
import java.util.Collections;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;
import java.util.TimeZone;

/* JADX INFO: Access modifiers changed from: package-private */
/* loaded from: com_mysql_jdbc.jar:com/mysql/jdbc/EscapeProcessor.class */
public class EscapeProcessor {
    private static Map JDBC_CONVERT_TO_MYSQL_TYPE_MAP;
    private static Map JDBC_NO_CONVERT_TO_MYSQL_EXPRESSION_MAP;

    EscapeProcessor() {
    }

    static {
        Map tempMap = new HashMap();
        tempMap.put("BIGINT", "0 + ?");
        tempMap.put("BINARY", "BINARY");
        tempMap.put("BIT", "0 + ?");
        tempMap.put("CHAR", "CHAR");
        tempMap.put("DATE", "DATE");
        tempMap.put("DECIMAL", "0.0 + ?");
        tempMap.put("DOUBLE", "0.0 + ?");
        tempMap.put("FLOAT", "0.0 + ?");
        tempMap.put("INTEGER", "0 + ?");
        tempMap.put("LONGVARBINARY", "BINARY");
        tempMap.put("LONGVARCHAR", "CONCAT(?)");
        tempMap.put("REAL", "0.0 + ?");
        tempMap.put("SMALLINT", "CONCAT(?)");
        tempMap.put("TIME", "TIME");
        tempMap.put("TIMESTAMP", "DATETIME");
        tempMap.put("TINYINT", "CONCAT(?)");
        tempMap.put("VARBINARY", "BINARY");
        tempMap.put("VARCHAR", "CONCAT(?)");
        JDBC_CONVERT_TO_MYSQL_TYPE_MAP = Collections.unmodifiableMap(tempMap);
        Map tempMap2 = new HashMap(JDBC_CONVERT_TO_MYSQL_TYPE_MAP);
        tempMap2.put("BINARY", "CONCAT(?)");
        tempMap2.put("CHAR", "CONCAT(?)");
        tempMap2.remove("DATE");
        tempMap2.put("LONGVARBINARY", "CONCAT(?)");
        tempMap2.remove("TIME");
        tempMap2.remove("TIMESTAMP");
        tempMap2.put("VARBINARY", "CONCAT(?)");
        JDBC_NO_CONVERT_TO_MYSQL_EXPRESSION_MAP = Collections.unmodifiableMap(tempMap2);
    }

    public static final Object escapeSQL(String sql, boolean serverSupportsConvertFn, ConnectionImpl conn) throws SQLException {
        String currentSql;
        Calendar sessionCalendar;
        Calendar sessionCalendar2;
        String remaining;
        boolean replaceEscapeSequence = false;
        String escapeSequence = null;
        if (sql == null) {
            return null;
        }
        int beginBrace = sql.indexOf(123);
        int nextEndBrace = beginBrace == -1 ? -1 : sql.indexOf(125, beginBrace);
        if (nextEndBrace == -1) {
            return sql;
        }
        StringBuffer newSql = new StringBuffer();
        EscapeTokenizer escapeTokenizer = new EscapeTokenizer(sql);
        byte usesVariables = 0;
        boolean callingStoredFunction = false;
        while (escapeTokenizer.hasMoreTokens()) {
            String token = escapeTokenizer.nextToken();
            if (token.length() != 0) {
                if (token.charAt(0) == '{') {
                    if (!token.endsWith("}")) {
                        throw SQLError.createSQLException(new StringBuffer().append("Not a valid escape sequence: ").append(token).toString());
                    }
                    if (token.length() > 2) {
                        int nestedBrace = token.indexOf(123, 2);
                        if (nestedBrace != -1) {
                            StringBuffer buf = new StringBuffer(token.substring(0, 1));
                            Object remainingResults = escapeSQL(token.substring(1, token.length() - 1), serverSupportsConvertFn, conn);
                            if (remainingResults instanceof String) {
                                remaining = (String) remainingResults;
                            } else {
                                remaining = ((EscapeProcessorResult) remainingResults).escapedSql;
                                if (usesVariables != 1) {
                                    usesVariables = ((EscapeProcessorResult) remainingResults).usesVariables;
                                }
                            }
                            buf.append(remaining);
                            buf.append('}');
                            token = buf.toString();
                        }
                    }
                    String collapsedToken = removeWhitespace(token);
                    if (StringUtils.startsWithIgnoreCase(collapsedToken, "{escape")) {
                        try {
                            StringTokenizer st = new StringTokenizer(token, " '");
                            st.nextToken();
                            escapeSequence = st.nextToken();
                            if (escapeSequence.length() < 3) {
                                newSql.append(token);
                            } else {
                                escapeSequence = escapeSequence.substring(1, escapeSequence.length() - 1);
                                replaceEscapeSequence = true;
                            }
                        } catch (NoSuchElementException e) {
                            newSql.append(token);
                        }
                    } else if (StringUtils.startsWithIgnoreCase(collapsedToken, "{fn")) {
                        String fnToken = token.substring(token.toLowerCase().indexOf("fn ") + 3, token.length() - 1);
                        if (StringUtils.startsWithIgnoreCaseAndWs(fnToken, "convert")) {
                            newSql.append(processConvertToken(fnToken, serverSupportsConvertFn));
                        } else {
                            newSql.append(fnToken);
                        }
                    } else if (StringUtils.startsWithIgnoreCase(collapsedToken, "{d")) {
                        int startPos = token.indexOf(39) + 1;
                        int endPos = token.lastIndexOf(39);
                        if (startPos == -1 || endPos == -1) {
                            newSql.append(token);
                        } else {
                            String argument = token.substring(startPos, endPos);
                            try {
                                StringTokenizer st2 = new StringTokenizer(argument, " -");
                                String dateString = new StringBuffer().append("'").append(st2.nextToken()).append("-").append(st2.nextToken()).append("-").append(st2.nextToken()).append("'").toString();
                                newSql.append(dateString);
                            } catch (NoSuchElementException e2) {
                                throw SQLError.createSQLException(new StringBuffer().append("Syntax error for DATE escape sequence '").append(argument).append("'").toString(), SQLError.SQL_STATE_SYNTAX_ERROR);
                            }
                        }
                    } else if (StringUtils.startsWithIgnoreCase(collapsedToken, "{ts")) {
                        int startPos2 = token.indexOf(39) + 1;
                        int endPos2 = token.lastIndexOf(39);
                        if (startPos2 == -1 || endPos2 == -1) {
                            newSql.append(token);
                        } else {
                            String argument2 = token.substring(startPos2, endPos2);
                            try {
                                StringTokenizer st3 = new StringTokenizer(argument2, " .-:");
                                String year4 = st3.nextToken();
                                String month2 = st3.nextToken();
                                String day2 = st3.nextToken();
                                String hour = st3.nextToken();
                                String minute = st3.nextToken();
                                String second = st3.nextToken();
                                if (!conn.getUseTimezone() && !conn.getUseJDBCCompliantTimezoneShift()) {
                                    newSql.append("'").append(year4).append("-").append(month2).append("-").append(day2).append(" ").append(hour).append(":").append(minute).append(":").append(second).append("'");
                                } else {
                                    if (conn != null) {
                                        sessionCalendar = conn.getCalendarInstanceForSessionOrNew();
                                    } else {
                                        sessionCalendar = new GregorianCalendar();
                                        sessionCalendar.setTimeZone(TimeZone.getTimeZone("GMT"));
                                    }
                                    try {
                                        int year4Int = Integer.parseInt(year4);
                                        int month2Int = Integer.parseInt(month2);
                                        int day2Int = Integer.parseInt(day2);
                                        int hourInt = Integer.parseInt(hour);
                                        int minuteInt = Integer.parseInt(minute);
                                        int secondInt = Integer.parseInt(second);
                                        synchronized (sessionCalendar) {
                                            boolean useGmtMillis = conn.getUseGmtMillisForDatetimes();
                                            Timestamp toBeAdjusted = TimeUtil.fastTimestampCreate(useGmtMillis, useGmtMillis ? Calendar.getInstance(TimeZone.getTimeZone("GMT")) : null, sessionCalendar, year4Int, month2Int, day2Int, hourInt, minuteInt, secondInt, 0);
                                            Timestamp inServerTimezone = TimeUtil.changeTimezone(conn, sessionCalendar, (Calendar) null, toBeAdjusted, sessionCalendar.getTimeZone(), conn.getServerTimezoneTZ(), false);
                                            newSql.append("'");
                                            String timezoneLiteral = inServerTimezone.toString();
                                            int indexOfDot = timezoneLiteral.indexOf(".");
                                            if (indexOfDot != -1) {
                                                timezoneLiteral = timezoneLiteral.substring(0, indexOfDot);
                                            }
                                            newSql.append(timezoneLiteral);
                                        }
                                        newSql.append("'");
                                    } catch (NumberFormatException e3) {
                                        throw SQLError.createSQLException(new StringBuffer().append("Syntax error in TIMESTAMP escape sequence '").append(token).append("'.").toString(), SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
                                    }
                                }
                            } catch (NoSuchElementException e4) {
                                throw SQLError.createSQLException(new StringBuffer().append("Syntax error for TIMESTAMP escape sequence '").append(argument2).append("'").toString(), SQLError.SQL_STATE_SYNTAX_ERROR);
                            }
                        }
                    } else if (StringUtils.startsWithIgnoreCase(collapsedToken, "{t")) {
                        int startPos3 = token.indexOf(39) + 1;
                        int endPos3 = token.lastIndexOf(39);
                        if (startPos3 == -1 || endPos3 == -1) {
                            newSql.append(token);
                        } else {
                            String argument3 = token.substring(startPos3, endPos3);
                            try {
                                StringTokenizer st4 = new StringTokenizer(argument3, " :");
                                String hour2 = st4.nextToken();
                                String minute2 = st4.nextToken();
                                String second2 = st4.nextToken();
                                if (!conn.getUseTimezone()) {
                                    String timeString = new StringBuffer().append("'").append(hour2).append(":").append(minute2).append(":").append(second2).append("'").toString();
                                    newSql.append(timeString);
                                } else {
                                    if (conn != null) {
                                        sessionCalendar2 = conn.getCalendarInstanceForSessionOrNew();
                                    } else {
                                        sessionCalendar2 = new GregorianCalendar();
                                    }
                                    try {
                                        int hourInt2 = Integer.parseInt(hour2);
                                        int minuteInt2 = Integer.parseInt(minute2);
                                        int secondInt2 = Integer.parseInt(second2);
                                        synchronized (sessionCalendar2) {
                                            Time toBeAdjusted2 = TimeUtil.fastTimeCreate(sessionCalendar2, hourInt2, minuteInt2, secondInt2);
                                            Time inServerTimezone2 = TimeUtil.changeTimezone(conn, sessionCalendar2, (Calendar) null, toBeAdjusted2, sessionCalendar2.getTimeZone(), conn.getServerTimezoneTZ(), false);
                                            newSql.append("'");
                                            newSql.append(inServerTimezone2.toString());
                                            newSql.append("'");
                                        }
                                    } catch (NumberFormatException e5) {
                                        throw SQLError.createSQLException(new StringBuffer().append("Syntax error in TIMESTAMP escape sequence '").append(token).append("'.").toString(), SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
                                    }
                                }
                            } catch (NoSuchElementException e6) {
                                throw SQLError.createSQLException(new StringBuffer().append("Syntax error for escape sequence '").append(argument3).append("'").toString(), SQLError.SQL_STATE_SYNTAX_ERROR);
                            }
                        }
                    } else if (StringUtils.startsWithIgnoreCase(collapsedToken, "{call") || StringUtils.startsWithIgnoreCase(collapsedToken, "{?=call")) {
                        int startPos4 = StringUtils.indexOfIgnoreCase(token, "CALL") + 5;
                        int endPos4 = token.length() - 1;
                        if (StringUtils.startsWithIgnoreCase(collapsedToken, "{?=call")) {
                            callingStoredFunction = true;
                            newSql.append("SELECT ");
                            newSql.append(token.substring(startPos4, endPos4));
                        } else {
                            callingStoredFunction = false;
                            newSql.append("CALL ");
                            newSql.append(token.substring(startPos4, endPos4));
                        }
                        int i = endPos4 - 1;
                        while (true) {
                            if (i >= startPos4) {
                                char c = token.charAt(i);
                                if (Character.isWhitespace(c)) {
                                    i--;
                                } else if (c != ')') {
                                    newSql.append("()");
                                }
                            }
                        }
                    } else if (StringUtils.startsWithIgnoreCase(collapsedToken, "{oj")) {
                        newSql.append(token);
                    }
                } else {
                    newSql.append(token);
                }
            }
        }
        String escapedSql = newSql.toString();
        if (replaceEscapeSequence) {
            String str = escapedSql;
            while (true) {
                currentSql = str;
                if (currentSql.indexOf(escapeSequence) == -1) {
                    break;
                }
                int escapePos = currentSql.indexOf(escapeSequence);
                String lhs = currentSql.substring(0, escapePos);
                String rhs = currentSql.substring(escapePos + 1, currentSql.length());
                str = new StringBuffer().append(lhs).append("\\").append(rhs).toString();
            }
            escapedSql = currentSql;
        }
        EscapeProcessorResult epr = new EscapeProcessorResult();
        epr.escapedSql = escapedSql;
        epr.callingStoredFunction = callingStoredFunction;
        if (usesVariables != 1) {
            if (escapeTokenizer.sawVariableUse()) {
                epr.usesVariables = (byte) 1;
            } else {
                epr.usesVariables = (byte) 0;
            }
        }
        return epr;
    }

    private static String processConvertToken(String functionToken, boolean serverSupportsConvertFn) throws SQLException {
        String newType;
        int firstIndexOfParen = functionToken.indexOf("(");
        if (firstIndexOfParen == -1) {
            throw SQLError.createSQLException(new StringBuffer().append("Syntax error while processing {fn convert (... , ...)} token, missing opening parenthesis in token '").append(functionToken).append("'.").toString(), SQLError.SQL_STATE_SYNTAX_ERROR);
        }
        functionToken.length();
        int indexOfComma = functionToken.lastIndexOf(",");
        if (indexOfComma == -1) {
            throw SQLError.createSQLException(new StringBuffer().append("Syntax error while processing {fn convert (... , ...)} token, missing comma in token '").append(functionToken).append("'.").toString(), SQLError.SQL_STATE_SYNTAX_ERROR);
        }
        int indexOfCloseParen = functionToken.indexOf(41, indexOfComma);
        if (indexOfCloseParen == -1) {
            throw SQLError.createSQLException(new StringBuffer().append("Syntax error while processing {fn convert (... , ...)} token, missing closing parenthesis in token '").append(functionToken).append("'.").toString(), SQLError.SQL_STATE_SYNTAX_ERROR);
        }
        String expression = functionToken.substring(firstIndexOfParen + 1, indexOfComma);
        String type = functionToken.substring(indexOfComma + 1, indexOfCloseParen);
        String trimmedType = type.trim();
        if (StringUtils.startsWithIgnoreCase(trimmedType, "SQL_")) {
            trimmedType = trimmedType.substring(4, trimmedType.length());
        }
        if (serverSupportsConvertFn) {
            newType = (String) JDBC_CONVERT_TO_MYSQL_TYPE_MAP.get(trimmedType.toUpperCase(Locale.ENGLISH));
        } else {
            newType = (String) JDBC_NO_CONVERT_TO_MYSQL_EXPRESSION_MAP.get(trimmedType.toUpperCase(Locale.ENGLISH));
            if (newType == null) {
                throw SQLError.createSQLException(new StringBuffer().append("Can't find conversion re-write for type '").append(type).append("' that is applicable for this server version while processing escape tokens.").toString(), SQLError.SQL_STATE_GENERAL_ERROR);
            }
        }
        if (newType == null) {
            throw SQLError.createSQLException(new StringBuffer().append("Unsupported conversion type '").append(type.trim()).append("' found while processing escape token.").toString(), SQLError.SQL_STATE_GENERAL_ERROR);
        }
        int replaceIndex = newType.indexOf("?");
        if (replaceIndex != -1) {
            StringBuffer convertRewrite = new StringBuffer(newType.substring(0, replaceIndex));
            convertRewrite.append(expression);
            convertRewrite.append(newType.substring(replaceIndex + 1, newType.length()));
            return convertRewrite.toString();
        }
        StringBuffer castRewrite = new StringBuffer("CAST(");
        castRewrite.append(expression);
        castRewrite.append(" AS ");
        castRewrite.append(newType);
        castRewrite.append(")");
        return castRewrite.toString();
    }

    private static String removeWhitespace(String toCollapse) {
        if (toCollapse == null) {
            return null;
        }
        int length = toCollapse.length();
        StringBuffer collapsed = new StringBuffer(length);
        for (int i = 0; i < length; i++) {
            char c = toCollapse.charAt(i);
            if (!Character.isWhitespace(c)) {
                collapsed.append(c);
            }
        }
        return collapsed.toString();
    }
}
